import { createLogger } from "dr-js/module/node/module/Logger";
import { addExitListenerAsync, addExitListenerSync } from "dr-js/module/node/system/ExitListener";
const EMPTY_FUNC = () => {};
const configureLogger = async ({pathLogDirectory, prefixLogFile}) => {
  const logger = (({add, ...logger}) => ({
    ...logger,
    add: (...args) => add(new Date().toISOString(), ...args)
  }))(pathLogDirectory ? await createLogger({
    pathLogDirectory,
    prefixLogFile,
    flags: "a"
  }) : {
    add: console.log,
    save: EMPTY_FUNC,
    split: EMPTY_FUNC,
    end: EMPTY_FUNC
  });
  addExitListenerAsync(exitState => {
    logger.add(`[EXITING] ${JSON.stringify(exitState)}`);
  });
  addExitListenerSync(exitState => {
    logger.add(`[EXIT] ${JSON.stringify(exitState)}`);
    exitState.error && logger.add(`[EXIT][ERROR] ${exitState.error.stack || exitState.error}`);
    logger.end();
  });
  return logger;
};
export { configureLogger };