import { clock, getTimestamp, createTimer } from "dr-js/module/common/time";
import { withRetryAsync, lossyAsync } from "dr-js/module/common/function";
import { roundFloat } from "dr-js/module/common/math/base";
import { fetch } from "dr-js/module/node/net";
import { createFactDatabase, tryDeleteExtraCache } from "dr-js/module/node/module/FactDatabase";
import { addExitListenerAsync, addExitListenerSync } from "dr-js/module/node/system/ExitListener";
import { applyFact } from "./applyStatusFact";
const configureStatusCollector = async ({collectPath: pathFactDirectory, collectUrl, collectInterval = 3e5, getExtraHeaders}) => {
  const factDB = await createFactDatabase({
    pathFactDirectory,
    applyFact,
    onError: error => console.error("[FactDatabase]", error)
  });
  addExitListenerSync(factDB.end);
  addExitListenerAsync(async () => {
    factDB.end();
    await factDB.getSaveFactCachePromise();
  });
  await tryDeleteExtraCache({
    pathFactDirectory
  });
  const collectStatus = lossyAsync(async () => {
    await withRetryAsync(async retryCount => {
      const timeFetchStart = clock();
      const {ok, json} = await fetch(collectUrl, {
        headers: getExtraHeaders ? getExtraHeaders() : {}
      });
      if (!ok) throw new Error("fetch no ok");
      const timeOk = roundFloat(clock() - timeFetchStart);
      const status = await json();
      const timeDownload = roundFloat(clock() - timeFetchStart - timeOk);
      factDB.add({
        timestamp: getTimestamp(),
        retryCount,
        status,
        timeOk,
        timeDownload
      });
    }, 3, 500);
  }, error => {
    console.error("[fetch] failed", collectUrl, error);
    factDB.add({
      timestamp: getTimestamp(),
      error: error.toString() || "fetch error"
    });
  }).trigger;
  const timer = createTimer({
    func: collectStatus,
    delay: collectInterval
  });
  return {
    factDB,
    timer,
    collectStatus
  };
};
export { configureStatusCollector };