import { setSumRaw, setRangeRaw, combineStatusRaw, combineStatus } from "./combine";
const floorTimestampByDay = timestamp => 86400 * Math.floor(timestamp / 86400);
const calcSystemMemoryUsed = ({free, total}) => (total - free) / total;
const calcSystemProcessorUsed = ({user, nice, sys, idle, irq}) => {
  const nonIdle = user + nice + sys + irq;
  return nonIdle / (idle + nonIdle);
};
const calcProcessProcessorUsed = ({user, system}, deltaTime) => .001 * (user + system) / deltaTime;
const splitChunk = (inputList, deltaTime, endTime, getTimeFunc, mergeFunc) => {
  if (!inputList.length) return [ inputList, [] ];
  const outputList = [];
  const indexMax = inputList.length;
  let index = 0;
  let chunkIndex = 0;
  let splitTime = Math.floor(getTimeFunc(inputList[0]) / deltaTime) * deltaTime;
  for (;index < indexMax; ) {
    const currentItemTime = getTimeFunc(inputList[index]);
    if (currentItemTime < splitTime) {
      chunkIndex !== index && outputList.push(mergeFunc(inputList.slice(chunkIndex, index)));
      chunkIndex = index;
      if (currentItemTime < endTime) break;
      splitTime = Math.max(Math.floor(currentItemTime / deltaTime) * deltaTime, endTime);
    }
    index++;
  }
  if (index !== chunkIndex) {
    outputList.push(mergeFunc(inputList.slice(chunkIndex, index)));
    chunkIndex = index;
  }
  return [ inputList.slice(0, chunkIndex), outputList ];
};
const applyFact = (state, fact) => {
  const {id, timestamp} = fact;
  1 === id && (state = (timestamp => ({
    id: 1,
    timestamp,
    sumKeyList: [],
    rangeKeyList: [],
    statusRawList: [],
    statusRawTimestamp: floorTimestampByDay(timestamp) + 86400,
    merge0List: [],
    merge0Timestamp: floorTimestampByDay(timestamp) + 604800,
    merge1List: [],
    merge1Timestamp: floorTimestampByDay(timestamp) + 3024e3,
    merge2List: []
  }))(timestamp));
  let {sumKeyList, rangeKeyList, statusRawList, statusRawTimestamp, merge0Timestamp, merge1Timestamp} = state;
  if (timestamp >= statusRawTimestamp) {
    const rangeTimestampIndex = rangeKeyList.indexOf("timestamp");
    const rawRangeValueIndex = 0;
    const getRawStatusTimeFunc = statusRaw => statusRaw.rangeRawList[rangeTimestampIndex][rawRangeValueIndex];
    const [clippedInputList, outputList] = splitChunk(statusRawList, 1200, statusRawTimestamp - 86400, getRawStatusTimeFunc, combineStatusRaw);
    statusRawList = clippedInputList;
    let merge0List = [ ...outputList, ...state.merge0List ];
    if (timestamp >= merge0Timestamp) {
      const rangeValueMaxIndex = 1;
      const getMergeStatusTimeFunc = mergeStatus => mergeStatus.rangeList[rangeTimestampIndex][rangeValueMaxIndex];
      const [clippedInputList, outputList] = splitChunk(merge0List, 3600, merge0Timestamp - 604800, getMergeStatusTimeFunc, combineStatus);
      merge0List = clippedInputList;
      let merge1List = [ ...outputList, ...state.merge1List ];
      if (timestamp >= merge1Timestamp) {
        const [clippedInputList, outputList] = splitChunk(merge1List, 86400, merge1Timestamp - 3024e3, getMergeStatusTimeFunc, combineStatus);
        merge1List = clippedInputList;
        let merge2List = [ ...outputList, ...state.merge2List ];
        merge2List.length > 256 && (merge2List.length = 256);
        state = {
          ...state,
          merge2List,
          merge1Timestamp: floorTimestampByDay(timestamp) + 3024e3
        };
      }
      state = {
        ...state,
        merge1List,
        merge0Timestamp: floorTimestampByDay(timestamp) + 604800
      };
    }
    state = {
      ...state,
      sumKeyList,
      rangeKeyList,
      merge0List,
      statusRawTimestamp: floorTimestampByDay(timestamp) + 86400
    };
  }
  const [statusRaw, nextSumKeyList, nextRangeKeyList] = (({timestamp, error, retryCount, status, timeOk, timeDownload}, sumKeyList, rangeKeyList) => {
    const sumRawList = [];
    const rangeRawList = [];
    rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "timestamp", timestamp);
    if (error) sumKeyList = setSumRaw(sumKeyList, sumRawList, "error", 1); else {
      const {timestamp: timestampStatus, systemStatus, processStatus, delta} = status;
      sumKeyList = setSumRaw(sumKeyList, sumRawList, "retryCount", retryCount);
      rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "timeOk", timeOk);
      rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "timeDownload", timeDownload);
      rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "timestampStatus", timestampStatus);
      rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "systemMemoryUsed", calcSystemMemoryUsed(systemStatus.memory));
      rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "processMemoryRSS", processStatus.memoryUsage.rss);
      rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "processMemoryHeap", processStatus.memoryUsage.heapTotal);
      delta.systemProcessorTimeList.forEach((systemProcessorTime, index) => {
        rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, `systemProcessorUsed[${index}]`, calcSystemProcessorUsed(systemProcessorTime), delta.time);
      });
      rangeKeyList = setRangeRaw(rangeKeyList, rangeRawList, "processProcessorUsed", calcProcessProcessorUsed(delta.processCpuUsage, delta.time), delta.time);
    }
    return [ {
      sumRawList,
      rangeRawList
    }, sumKeyList, rangeKeyList ];
  })(fact, sumKeyList, rangeKeyList);
  return {
    ...state,
    id,
    timestamp,
    sumKeyList: nextSumKeyList,
    rangeKeyList: nextRangeKeyList,
    statusRawList: [ statusRaw, ...statusRawList ]
  };
};
export { applyFact };