import { roundFloat } from "dr-js/module/common/math/base";
import { isNumber, isBasicArray } from "dr-js/module/common/check";
const INITIAL_MERGE_VALUE = [ 1 / 0, -1 / 0, 0, 0 ];
const setSumRaw = (sumKeyList, sumRawList, key, value) => {
  let index = sumKeyList.indexOf(key);
  if (-1 === index) {
    index = sumKeyList.length;
    sumKeyList = [ ...sumKeyList, key ];
  }
  sumRawList[index] = roundFloat(value);
  return sumKeyList;
};
const setRangeRaw = (rangeKeyList, rangeRawList, key, value, valueSize = 1) => {
  let index = rangeKeyList.indexOf(key);
  if (-1 === index) {
    index = rangeKeyList.length;
    rangeKeyList = [ ...rangeKeyList, key ];
  }
  rangeRawList[index] = [ roundFloat(value), roundFloat(valueSize) ];
  return rangeKeyList;
};
const combineStatusRaw = statusRawList => {
  const sumList = [];
  const rangeList = [];
  statusRawList.forEach(({sumRawList, rangeRawList}) => {
    sumRawList.forEach((sumRaw, index) => {
      isNumber(sumRaw) && (sumList[index] = (sumList[index] || 0) + sumRaw);
    });
    rangeRawList.forEach((rangeRaw, index) => {
      if (!isBasicArray(rangeRaw)) return;
      const [value, valueSize] = rangeRaw;
      const [min, max, sum, size] = rangeList[index] || INITIAL_MERGE_VALUE;
      rangeList[index] = [ Math.min(min, value), Math.max(max, value), roundFloat(sum + value * valueSize), roundFloat(size + valueSize) ];
    });
  });
  return {
    size: statusRawList.length,
    sumList,
    rangeList
  };
};
const combineStatus = mergeStatusList => {
  let size = 0;
  const sumList = [];
  const rangeList = [];
  mergeStatusList.forEach(mergeStatus => {
    size += mergeStatus.size;
    mergeStatus.sumList.forEach((value, index) => {
      isNumber(value) && (sumList[index] = (sumList[index] || 0) + value);
    });
    mergeStatus.rangeList.forEach((mergeValue, index) => {
      if (!isBasicArray(mergeValue)) return;
      const [min, max, sum, size] = rangeList[index] || INITIAL_MERGE_VALUE;
      rangeList[index] = [ Math.min(min, mergeValue[0]), Math.max(max, mergeValue[1]), roundFloat(sum + mergeValue[2]), roundFloat(size + mergeValue[3]) ];
    });
  });
  return {
    size,
    sumList,
    rangeList
  };
};
export { setSumRaw, setRangeRaw, combineStatusRaw, combineStatus };