import { relative } from "path";
import { catchAsync } from "dr-js/module/common/error";
import { createPathPrefixLock, toPosixPath } from "dr-js/module/node/file/function";
import { createDirectory } from "dr-js/module/node/file/File";
import { getDirectorySubInfoList, getDirectoryInfoTree, walkDirectoryInfoTree } from "dr-js/module/node/file/Directory";
import { modify } from "dr-js/module/node/file/Modify";
const createGetPathModify = rootPath => {
  const getPath = createPathPrefixLock(rootPath);
  const pathModify = {
    ...modify,
    "create-directory": absolutePath => createDirectory(absolutePath),
    "path-content": async absolutePath => {
      const {result: subInfoList, error} = await catchAsync(getDirectorySubInfoList, absolutePath);
      const directoryList = [];
      const fileList = [];
      subInfoList && subInfoList.forEach(({name, stat}) => stat.isDirectory() ? directoryList.push(name) : fileList.push([ name, stat.size, Math.round(stat.mtimeMs) ]));
      return {
        directoryList,
        fileList
      };
    },
    "list-file-recursive": async absolutePath => {
      const fileList = [];
      const {error} = await catchAsync(async () => walkDirectoryInfoTree(await getDirectoryInfoTree(absolutePath), ({path, stat}) => !stat.isDirectory() && fileList.push([ toPosixPath(relative(absolutePath, path)), stat.size, Math.round(stat.mtimeMs) ])));
      return {
        fileList
      };
    }
  };
  return async (modifyType, relativePathFrom, relativePathTo) => {
    const absolutePathFrom = getPath(relativePathFrom);
    const absolutePathTo = relativePathTo && getPath(relativePathTo);
    return {
      modifyType,
      relativePathFrom,
      relativePathTo,
      ...await pathModify[modifyType](absolutePathFrom, absolutePathTo)
    };
  };
};
export { createGetPathModify };