import { clock, getTimestamp } from "dr-js/module/common/time";
import { binary } from "dr-js/module/common/format";
import { roundFloat } from "dr-js/module/common/math/base";
import { getSystemProcessor, getSystemStatus, getProcessStatus } from "dr-js/module/node/system/Status";
const createGetStatusReport = (processTag = (({platform, processor, memory, network} = getSystemStatus()) => [ platform.platform, platform.arch, platform.release, processor.length, processor[0].model, processor[0].speed, binary(memory.total), network.hostname ].filter(Boolean).join("|"))()) => {
  let prevTime = clock();
  let prevSystemStatus = getSystemStatus();
  let prevProcessStatus = getProcessStatus();
  return () => {
    const time = clock();
    const systemStatus = getSystemStatus();
    const processStatus = getProcessStatus();
    const statusReport = {
      timestamp: getTimestamp(),
      processTag,
      systemStatus,
      processStatus,
      delta: {
        time: roundFloat(time - prevTime),
        systemProcessorTimeList: ((status = getSystemProcessor(), prevStatus) => status.map(({times: {user, nice, sys, idle, irq}}, index) => {
          const prevTimes = prevStatus[index].times;
          return {
            user: user - prevTimes.user,
            nice: nice - prevTimes.nice,
            sys: sys - prevTimes.sys,
            idle: idle - prevTimes.idle,
            irq: irq - prevTimes.irq
          };
        }))(systemStatus.processor, prevSystemStatus.processor),
        processCpuUsage: (({user, system} = process.cpuUsage(), prevValue) => ({
          user: user - prevValue.user,
          system: system - prevValue.system
        }))(processStatus.cpuUsage, prevProcessStatus.cpuUsage)
      }
    };
    prevTime = time;
    prevSystemStatus = systemStatus;
    prevProcessStatus = processStatus;
    return statusReport;
  };
};
export { createGetStatusReport };