var _Buffer = require("dr-js/library/node/data/Buffer");
var _Server = require("dr-js/library/node/server/Server");
var _Common = require("dr-js/library/node/server/Responder/Common");
var _Send = require("dr-js/library/node/server/Responder/Send");
var _Router = require("dr-js/library/node/server/Responder/Router");
var _logger = require("../library/configure/logger");
var _filePid = require("../library/configure/filePid");
var _auth = require("../library/configure/auth");
var _serverBase = require("../library/configure/serverBase");
var _Explorer = require("../library/responder/pathContent/Explorer");
exports.createServer = (async ({filePid, shouldIgnoreExistPid, protocol, hostname, port, fileSSLKey, fileSSLCert, fileSSLChain, fileSSLDHParam, pathLogDirectory, prefixLogFile, fileAuthConfig, shouldAuthGen, authGenTag, authGenSize, authGenTokenSize, authGenTimeGap, uploadRootPath, uploadMergePath}) => {
  await (0, _filePid.configureFilePid)({
    filePid,
    shouldIgnoreExistPid
  });
  const {server, start, stop, option} = await (0, _serverBase.configureServerBase)({
    protocol,
    hostname,
    port,
    fileSSLKey,
    fileSSLCert,
    fileSSLChain,
    fileSSLDHParam
  });
  const logger = await (0, _logger.configureLogger)({
    pathLogDirectory,
    prefixLogFile
  });
  const {wrapResponderCheckAuthCheckCode} = await (0, _auth.configureAuthTimedLookup)({
    fileAuthConfig,
    shouldAuthGen,
    authGenTag,
    authGenSize,
    authGenTokenSize,
    authGenTimeGap,
    logger
  });
  const responderLogEnd = (0, _Common.createResponderLogEnd)(logger.add);
  const responderFileChunkUpload = await (0, _Explorer.createResponderFileChunkUpload)({
    rootPath: uploadRootPath,
    mergePath: uploadMergePath,
    onError: error => {
      logger.add(`[ERROR] ${error}`);
      console.error(error);
    }
  });
  const responderPathModify = (0, _Explorer.createResponderPathModify)(uploadRootPath);
  const responderPathBatchModify = (0, _Explorer.createResponderPathBatchModify)(uploadRootPath);
  const responderServeFile = (0, _Explorer.createResponderServeFile)(uploadRootPath);
  const responderStorageStatus = (0, _Explorer.createResponderStorageStatus)(uploadRootPath);
  const routerMap = (0, _Router.createRouteMap)([ [ "/explorer", "GET", await (0, _Explorer.createResponderExplorer)({
    urlAuthCheck: "/auth",
    urlPathModify: "/path-modify",
    urlPathBatchModify: "/path-batch-modify",
    urlFileUpload: "/file-chunk-upload",
    urlFileServe: "/file-serve",
    urlStorageStatus: "/storage-status"
  }) ], [ "/path-modify", "POST", wrapResponderCheckAuthCheckCode(async store => {
    const {modifyType, relativePathFrom, relativePathTo} = JSON.parse(await (0, _Buffer.receiveBufferAsync)(store.request));
    return responderPathModify(store, modifyType, relativePathFrom, relativePathTo);
  }) ], [ "/path-batch-modify", "POST", wrapResponderCheckAuthCheckCode(async store => {
    const {nameList, modifyType, relativePathFrom, relativePathTo} = JSON.parse(await (0, _Buffer.receiveBufferAsync)(store.request));
    return responderPathBatchModify(store, nameList, modifyType, relativePathFrom, relativePathTo);
  }) ], [ "/file-chunk-upload", "POST", wrapResponderCheckAuthCheckCode(responderFileChunkUpload) ], [ "/storage-status", "GET", wrapResponderCheckAuthCheckCode(responderStorageStatus) ], [ "/file-serve/*", "GET", wrapResponderCheckAuthCheckCode(store => responderServeFile(store, decodeURIComponent((0, 
  _Router.getRouteParamAny)(store)))) ], [ "/auth", "GET", wrapResponderCheckAuthCheckCode(store => (0, _Common.responderEndWithStatusCode)(store, {
    statusCode: 200
  })) ], [ "/", "GET", store => (0, _Common.responderEndWithRedirect)(store, {
    redirectUrl: "/explorer"
  }) ], [ [ "/favicon", "/favicon.ico" ], "GET", (0, _Send.createResponderFavicon)() ] ].filter(Boolean));
  server.on("request", (0, _Server.createRequestListener)({
    responderList: [ (0, _Common.createResponderParseURL)(option), (0, _Common.createResponderLog)(logger.add), (0, _Router.createResponderRouter)(routerMap) ],
    responderEnd: store => {
      (0, _Common.responderEnd)(store);
      responderLogEnd(store);
    }
  }));
  return {
    server,
    start,
    stop,
    option,
    logger
  };
});