var _time = require("dr-js/library/common/time");
var _base = require("dr-js/library/common/math/base");
var _Buffer = require("dr-js/library/node/data/Buffer");
var _Server = require("dr-js/library/node/server/Server");
var _Common = require("dr-js/library/node/server/Responder/Common");
var _Send = require("dr-js/library/node/server/Responder/Send");
var _Router = require("dr-js/library/node/server/Responder/Router");
var _logger = require("../library/configure/logger");
var _filePid = require("../library/configure/filePid");
var _auth = require("../library/configure/auth");
var _serverBase = require("../library/configure/serverBase");
var _Collector = require("../library/configure/status/Collector");
var _Visualize = require("../library/responder/status/Visualize");
exports.createServer = (async ({filePid, shouldIgnoreExistPid, protocol, hostname, port, fileSSLKey, fileSSLCert, fileSSLChain, fileSSLDHParam, pathLogDirectory, prefixLogFile, fileAuthConfig, shouldAuthGen, authGenTag, authGenSize, authGenTokenSize, authGenTimeGap, statusCollectPath, statusCollectUrl, statusCollectInterval}) => {
  await (0, _filePid.configureFilePid)({
    filePid,
    shouldIgnoreExistPid
  });
  const {server, start, stop, option} = await (0, _serverBase.configureServerBase)({
    protocol,
    hostname,
    port,
    fileSSLKey,
    fileSSLCert,
    fileSSLChain,
    fileSSLDHParam
  });
  const logger = await (0, _logger.configureLogger)({
    pathLogDirectory,
    prefixLogFile
  });
  const {generateAuthCheckCode, wrapResponderCheckAuthCheckCode} = await (0, _auth.configureAuthTimedLookup)({
    fileAuthConfig,
    shouldAuthGen,
    authGenTag,
    authGenSize,
    authGenTokenSize,
    authGenTimeGap,
    logger
  });
  const {factDB, timer} = await (0, _Collector.configureStatusCollector)({
    collectPath: statusCollectPath,
    collectUrl: statusCollectUrl,
    collectInterval: statusCollectInterval,
    getExtraHeaders: () => ({
      "auth-check-code": generateAuthCheckCode()
    })
  });
  const responderLogEnd = (0, _Common.createResponderLogEnd)(logger.add);
  const responderAuthCheck = wrapResponderCheckAuthCheckCode(store => (0, _Common.responderEndWithStatusCode)(store, {
    statusCode: 200
  }));
  const routerMap = (0, _Router.createRouteMap)([ [ "/status-visualize", "GET", await (0, _Visualize.createResponderStatusVisualize)("/status-state", "/auth") ], [ "/status-state", "GET", wrapResponderCheckAuthCheckCode((0, _Visualize.createResponderStatusState)(factDB.getState)) ], [ "/status-collect", "POST", wrapResponderCheckAuthCheckCode(async store => {
    const statusBuffer = await (0, _Buffer.receiveBufferAsync)(store.request);
    factDB.add({
      timestamp: (0, _time.getTimestamp)(),
      retryCount: 0,
      status: JSON.parse(statusBuffer),
      timeOk: 0,
      timeDownload: (0, _base.roundFloat)((0, _time.clock)() - store.getState().time)
    });
    (0, _Common.responderEndWithStatusCode)(store, {
      statusCode: 200
    });
  }) ], [ "/auth", "GET", responderAuthCheck ], [ "/", "GET", store => (0, _Common.responderEndWithRedirect)(store, {
    redirectUrl: "/status-visualize"
  }) ], [ [ "/favicon", "/favicon.ico" ], "GET", (0, _Send.createResponderFavicon)() ] ]);
  server.on("request", (0, _Server.createRequestListener)({
    responderList: [ (0, _Common.createResponderParseURL)(option), (0, _Common.createResponderLog)(logger.add), (0, _Router.createResponderRouter)(routerMap) ],
    responderEnd: store => {
      (0, _Common.responderEnd)(store);
      responderLogEnd(store);
    }
  }));
  return {
    server,
    start,
    stop,
    option,
    logger,
    timer
  };
});