var _net = require("dr-js/library/node/net");
var _Server = require("dr-js/library/node/server/Server");
var _Common = require("dr-js/library/node/server/Responder/Common");
var _Send = require("dr-js/library/node/server/Responder/Send");
var _Router = require("dr-js/library/node/server/Responder/Router");
var _logger = require("../library/configure/logger");
var _filePid = require("../library/configure/filePid");
var _auth = require("../library/configure/auth");
var _serverBase = require("../library/configure/serverBase");
var _Report = require("../library/responder/status/Report");
var _getStatusReport = require("../library/task/getStatusReport");
exports.createServer = (async ({filePid, shouldIgnoreExistPid, protocol, hostname, port, fileSSLKey, fileSSLCert, fileSSLChain, fileSSLDHParam, pathLogDirectory, prefixLogFile, fileAuthConfig, shouldAuthGen, authGenTag, authGenSize, authGenTokenSize, authGenTimeGap, statusReportProcessTag}) => {
  await (0, _filePid.configureFilePid)({
    filePid,
    shouldIgnoreExistPid
  });
  const {server, start, stop, option} = await (0, _serverBase.configureServerBase)({
    protocol,
    hostname,
    port,
    fileSSLKey,
    fileSSLCert,
    fileSSLChain,
    fileSSLDHParam
  });
  const logger = await (0, _logger.configureLogger)({
    pathLogDirectory,
    prefixLogFile
  });
  const {generateAuthCheckCode, wrapResponderCheckAuthCheckCode} = await (0, _auth.configureAuthTimedLookup)({
    fileAuthConfig,
    shouldAuthGen,
    authGenTag,
    authGenSize,
    authGenTokenSize,
    authGenTimeGap,
    logger
  });
  const responderLogEnd = (0, _Common.createResponderLogEnd)(logger.add);
  const routerMap = (0, _Router.createRouteMap)([ [ "/status-report", "GET", wrapResponderCheckAuthCheckCode((0, _Report.createResponderStatusReport)(statusReportProcessTag)) ], [ "/", "GET", store => (0, _Common.responderEndWithRedirect)(store, {
    redirectUrl: "/status-report"
  }) ], [ [ "/favicon", "/favicon.ico" ], "GET", (0, _Send.createResponderFavicon)() ] ]);
  server.on("request", (0, _Server.createRequestListener)({
    responderList: [ (0, _Common.createResponderParseURL)(option), (0, _Common.createResponderLog)(logger.add), (0, _Router.createResponderRouter)(routerMap) ],
    responderEnd: store => {
      (0, _Common.responderEnd)(store);
      responderLogEnd(store);
    }
  }));
  const getStatusReport = (0, _getStatusReport.createGetStatusReport)(statusReportProcessTag);
  return {
    server,
    start,
    stop,
    option,
    logger,
    reportStatus: async url => {
      const {status} = await (0, _net.fetch)(url, {
        method: "POST",
        headers: {
          "auth-check-code": generateAuthCheckCode()
        },
        body: JSON.stringify(getStatusReport())
      });
    }
  };
});